//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


/*****************************************************************/
/*                                                               */
/*  Class AdaBoostMH                                             */
/*                                                               */
/*****************************************************************/

#ifndef _ADABOOST
#define _ADABOOST

#include "dataset.h"
#include <fstream>
#include <iostream>
#include <string>
#include <vector>

// defined below
class mlABTree; 

class adaboost {
private:
  // class parameters
  static double epsilon;
  static int    verbose;
  static bool   option_initialize_weights;


  struct wr_holder {
    mlABTree *rule;
    wr_holder  *next;
  };


  // weakrules linked list
  wr_holder  *first;
  wr_holder  *last;
  wr_holder  *pcl_pointer; // partial classification pointer
  int         nrules;
  int         nlabels;   
  vector<string> labels;  //label names, loaded from model file
  string         label_others;

  // output 
  ofstream    *out;

  // stopping criterion
  struct {
    int  n_rounds;
    int  max_depth;
  } SC;

  // auxiliar learning functions
  int  stopping_criterion(int nrounds);
  void initialize_weights(dataset *ds);
  void update_weights(mlABTree *wr, double Z, dataset *ds);
  void add_weak_rule(mlABTree *wr);

  // copy constructor forbidden
  adaboost(const adaboost &old_bab); 

public:
  // constructors, destructor and access methods
  adaboost(int nl);
  adaboost(const string &file);
  ~adaboost();
  int n_rules();

  int get_nlabels() {return nlabels;}
  string get_label(int lb) {return labels[lb];}
  string default_class() {return label_others;}

  // classification methods
  // Important: pred is an array of predictions, one for each label
  //            the function *assigns* its predicion for each label
  void classify(input *i,  double pred[]);

  // partial classification
  void pcl_ini_pointer();
  int  pcl_advance_pointer(int steps);
  // Important: pred is an array of predictions, one for each label
  //            the function *adds* its predicion for each label
  void pcl_classify(input *i, double *pred, int nrules);

  // learning methods
  void learn(dataset *ds, int nrounds, int maxdepth);


  // I/O methods
  void set_output(ofstream &os);
  void read_from_stream(ifstream &in);
  void read_from_file(char* file);

  static void set_verbose(int level);
  static void set_epsilon(double eps);
  static void set_initialize_weights(bool b);
};



/*****************************************************************/
/*                                                               */
/*  Class mlABTree                                               */
/*                                                               */
/*****************************************************************/


class mlABTree {

private:
  // binary tree structure
  int         feature;        // 0 when leaf
  mlABTree  **sons;           // when no leaf 
  double     *predictions;    // when leaf  (array of predicitons, one for each class)

  // learning parameters
  static int       nlabels;
  static double    epsilon;
  static int       max_depth;
  static int      *used_features; 
  static int       verbose;

  // auxiliar learning functions
  static mlABTree* learn_0(dataset *ds, double *Z, int depth);
  static int       stopping_criterion(dataset *ds, int depth);
  // W is W[2][nlabels][2]
  static int       best_feature(dataset *ds, double *W);

  // W is W[ndim][nlabels][2]
  static double    Zcalculus(double *W, int ndim);
  // W is W[v][nlabels][2]; result is result[nlabels][2]
  static void      Cprediction(int v, double *W, double result[]);

  // copy constructor forbidden
  mlABTree(const mlABTree &wr0);

public:
  // class parameters
  static void set_nlabels(int nl);
  static void set_verbose(int level);
  static void set_epsilon(double eps);

  // Constructors and destructor

  //  p0 is an array of predicions, one for each label
  mlABTree(double *p0);
  mlABTree(int f, mlABTree *wrFalse, mlABTree *wrTrue);
  ~mlABTree();

  // Classification
  // Important: pred is an array of predictions, one for each label
  //            the function *adds* its predicion for each label
  void classify(input *i,double *pred);

  //  I/O operations
  void print(char *carry);
  void write_to_stream(ofstream &os);
  static mlABTree* read_from_stream(istream &is);

  // learning
  static mlABTree* learn(dataset *ds, double *Z, int max_depth0);
};



#endif 
