//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


#ifndef _DEPRULES
#define _DEPRULES

#include <set>

using namespace std;



////////////////////////////////////////////////////////////////
///
///  The class completerRule stores rules used by the
/// completer of parse trees
///
////////////////////////////////////////////////////////////////

class completerRule {
 
 public:
   string chunk;
   string leftChunk;
   string newNode;
   string operation;
   string dependenceLabel;
   int weight;
   
   /// constructors
   completerRule();
   completerRule(const string &, const string&);
   completerRule( const completerRule &);
   /// assignment
   completerRule & operator=( const completerRule &);
   
   /// Comparison. The more weight the higher priority 
   int operator<(const completerRule & a ) const;
};


////////////////////////////////////////////////////////////////
///
/// The class rule_expression is an abstract class (interface)
/// for building dynamic restriction on a ruleLabeler 
///  which are used by class depLabeler
///
////////////////////////////////////////////////////////////////
class rule_expression {

  protected:
    // set of values (if any) to check against.
    set<string> valueList;

  public:
    // constructors and destructors
    rule_expression();
    rule_expression(const string &);
    virtual ~rule_expression() {}
    // search a value in expression list
    bool find(const string &) const;
    // virtual, evaluate expression
    virtual bool eval(dep_tree::iterator, dep_tree::iterator) const =0;
};


/////////////////////////////////////////////////////////////////
///
/// The following classes are implementations of different constraints
///
/// check_and:  and of basic constraints
/// check_not:  negation
/// check_side: side (left/rigth) of descendant wrt ancestor 
/// check_A_lemma: ancestor lemma in list of lemmas (separator character is |)
/// check_S_lemma: descendant lemma in list of lemmas (separator character is |)
/// check_S_category: ancestor category
/// check_A_wordclass: ancestor lemma belongs to a word class
///
/////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
/// And of basic contraints
////////////////////////////////////////////////////////////////

class check_and : public rule_expression {
  private:
    list<rule_expression *> check_list;
  public:
    void add(rule_expression *);
    bool eval(dep_tree::iterator  ancestor, dep_tree::iterator  descendant) const;
};


////////////////////////////////////////////////////////////////
/// negation
////////////////////////////////////////////////////////////////

class check_not : public rule_expression {
  private:
    rule_expression * check_op;
  public:
    check_not(rule_expression *);
    bool eval(dep_tree::iterator, dep_tree::iterator) const;
};


////////////////////////////////////////////////////////////////
///  descendant is to the right of its ancestor 
////////////////////////////////////////////////////////////////

class check_side : public rule_expression {
  private:
    string side;
  public:
    check_side(const string &);
    bool eval (dep_tree::iterator, dep_tree::iterator) const;
    static rule_expression * create(const string &);
};


////////////////////////////////////////////////////////////////
/// ancestor lemma in list of lemmas (separator character is |)
////////////////////////////////////////////////////////////////

class check_A_lemma :public rule_expression {
  public:
    check_A_lemma(const string &);
    bool eval(dep_tree::iterator, dep_tree::iterator) const;
    static rule_expression* create(const string &);
};

////////////////////////////////////////////////////////////////
/// descendant lemma in list of lemmas (separator character is |)
////////////////////////////////////////////////////////////////

class check_S_lemma :public rule_expression {
  public:
    check_S_lemma(const string &);
    bool eval(dep_tree::iterator, dep_tree::iterator) const;
    static rule_expression* create(const string &);
};


////////////////////////////////////////////////////////////////
/// ancestor category
////////////////////////////////////////////////////////////////

class check_S_category :public rule_expression {
  private:
    string catg;
  public:
    check_S_category(const string &);
    bool eval(dep_tree::iterator, dep_tree::iterator) const;
    static rule_expression* create(const string &);
};


////////////////////////////////////////////////////////////////
/// ancestor lemma belongs to a verb class
////////////////////////////////////////////////////////////////

class check_A_wordclass : public rule_expression { 
  private: 
    string wclass;
  public:    
    static set<string> wordclasses; // items are class#verbLemma
    check_A_wordclass(const string &);
    static rule_expression * create(const string &);
    bool eval (dep_tree::iterator, dep_tree::iterator) const;
};


////////////////////////////////////////////////////////////////
/// ruleLabeler is an auxiliary class for the depLabeler
////////////////////////////////////////////////////////////////

class ruleLabeler {

  public:
   string label;
   rule_expression * re;
   string ancestorLabel;

   ruleLabeler(void);
   ruleLabeler(const string &, rule_expression *);
   bool eval(dep_tree::iterator, dep_tree::iterator) const;
};


////////////////////////////////////////////////////////////////
/// auxiliary class to store information about dependency building
////////////////////////////////////////////////////////////////

class dep_info {

  private:
    string dep_source;
    string dep_target;
    string dep_result;

  public:
    dep_info(void);
    dep_info(const string &, const string &, const string &);
    
    string get_dep_source(void);
    string get_dep_target(void);
    string get_dep_result(void);
    void set_dep_source(const string &);
    void set_dep_target(const string &);
    void set_dep_result(const string &);
};


#endif
