//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


#ifndef _DEPENDENCIES
#define _DEPENDENCIES

using namespace std;

#include "language.h"
#include "dep_rules.h"

#include <string>
#include <map>


////////////////////////////////////////////////////////////////
///
///  The class completer implements a parse tree completer,
/// which given a partial parse tree (chunker output), completes
/// the full parse according to some grammar rules.
///
////////////////////////////////////////////////////////////////

class completer {

  private:
    /// set of rules, indexed by labels of nodes
    map<pair<string,string>,completerRule>  chgram;
    /// find last node with given label in a subtree
    parse_tree::iterator find_last_label(parse_tree *, const string &) const;
    /// retrieve rule from grammar
    completerRule gramatica(const string &, const string &);
    /// apply a completion rule
    parse_tree * applyRule(const completerRule &, parse_tree *, parse_tree *);

  public:  
    /// Constructor. Load a tree-completion grammar 
    completer(const string &);
    /// complete given parse tree
    parse_tree complete(parse_tree &, const string &);    
};



/// expression creator generic function, to allow registering new expressions

typedef rule_expression* expcreator(const string &);

////////////////////////////////////////////////////////////////////////
///
/// depLabeler is class to set labels into a dependency tree
///
///////////////////////////////////////////////////////////////////////

class depLabeler {

  private:
    map<string, list <ruleLabeler> > rules;
    map<string, expcreator *> expressions;

  public:
    /// Constructor. Load dependency labelling rules
    depLabeler(const string &);
    /// Label nodes in a depencendy tree. (Initial call)
    void label(dep_tree*);
    /// Label nodes in a depencendy tree. (recursive)
    void label(dep_tree*, dep_tree::iterator);
    /// register a new tree joining operation to be used in rule file
    void registerFunction(const string, expcreator*);
};



///////////////////////////////////////////////////////////////////////
///
/// dependencyMaker is a class for obtaining a dependency tree from chunks.
///  this implementation uses two subclasses:
/// completer: to complete the chunk analysis in a full parse tree
/// depLabeler: to set the labels once the class has build a dependency tree
///
///////////////////////////////////////////////////////////////////////

class dependencyMaker {

 private:
   /// tree completer  
   completer comp; 
   /// dependency labeler
   depLabeler labeler;
   // Root symbol used by the chunk parser when the tree is not complete.
   string start;
  
   /// register a new tree joining operation
   void registerFunction(const string &, expcreator*);
   /// compute dependency tree
   dep_tree* dependencies(parse_tree &tr);

 public:   
   /// constructor
   dependencyMaker(const string &, const string &);
   /// Enrich all sentences in given list with a depenceny tree.
   void analyze(list<sentence> &);
   /// Enrich all sentences in given list, return a copy.
   list<sentence> analyze(const list<sentence> &);
};

#endif

