//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


#ifndef _EXAMPLE
#define _EXAMPLE

#include <string>
#include <map>
#include <list>
#include <set>

using namespace std;


/********************************************************************************/
/*                                                                              */
/*  iFeature:  a feature with integer label and real value                      */
/*                                                                              */
/********************************************************************************/

class iFeature {
 private:
  int _label;
  double _value;
  
 public:
  iFeature(int l, double v = 1.0)
    : _label(l), _value(v)
    {}

  int label() const { return _label; }
  double value() const { return _value; }
  void set_value(double v) { _value = v; }

  bool operator<(const iFeature& rhs) const { return _label < rhs._label; }

};



/********************************************************************************/
/*                                                                              */
/*  input:  vector of features                                                  */
/*                                                                              */
/********************************************************************************/

class map_input;
class set_input;

typedef map_input input;

class map_input {
 private:
  map<int,iFeature> mf;
  int _dimension;

  static bool  consider_value;
  static char label_value_separator;

 public:

  // static functions
  static void set_consider_value (bool);
  static void set_label_value_separator (char);
  
  // constructors & destructor
  map_input()
    : _dimension(0)
    {}

  // string format: <l1>:<v1> <l2>:<v2> <ldim>:<vdim>
  map_input(string);

  // new input is f1*i1 + f2*i2
  map_input(double f1, const map_input& i1, double f2, const map_input& i2);

  ~map_input();

  // update functions
  void add_feature(const iFeature& f);
  void add_feature(int l, double v = 1.0);

  // string format: <l1>:<v1> <l2>:<v2> <ldim>:<vdim>
  void parse_string(string);

  // consultors
  double feature_value(int label) const;
  int    size() const { return mf.size(); }
  int    dimension() const { return _dimension; }

  // calcul
  double inner_product(map_input*) const;

  // recorregut de features
  // typedef  map<int,iFeature*>::iterator iterator;
  class const_iterator {
    private :
      map<int,iFeature>::const_iterator i;

    public:
    const_iterator() 
      {};
    const_iterator(map<int,iFeature>::const_iterator i0) 
      : i(i0)
      {}

    const iFeature* operator->() const { return &(i->second); }
    
    const_iterator& operator++() { ++i; return *this; }
    
    bool operator==(const const_iterator& rhs) { return i == rhs.i; }
    bool operator!=(const const_iterator& rhs) { return i != rhs.i; }
    
  };

  const_iterator begin() const { const_iterator i(mf.begin()); return i; }
  const_iterator end() const { const_iterator i(mf.end()); return i; }
  

  void print(ostream& o) const;
  void write(ostream& o) const;
};



class set_input {
 private:
  set<iFeature> sf;
  int _dimension;

  static bool  consider_value;
  static char label_value_separator;

 public:

  // static functions
  static void set_consider_value (bool);
  static void set_label_value_separator (char);
  
  // constructors & destructor
  set_input()
    : _dimension(0)
    {}

  // string format: <l1>:<v1> <l2>:<v2> <ldim>:<vdim>
  set_input(string);

  // string format, char* parameter for SWIG compatibility
  set_input(char* s) 
    : _dimension(0)
    { string s0(s); parse_string(s0); }

  // copy constructor
  set_input(const set_input& i1);

  // new input is f1*i1 + f2*i2
  set_input(double f1, const set_input& i1, double f2, const set_input& i2);

  ~set_input();

  // update functions
  void add_feature(const iFeature& f);
  void add_feature(int l, double v = 1.0);

  void multiply_by_factor(double f); 
  void normalize(); 

  // string format: <l1>:<v1> <l2>:<v2> <ldim>:<vdim>
  void parse_string(string);

  // consultors
  double feature_value(int label) const;
  int    size() const { return sf.size(); }
  int    dimension() const { return _dimension; }

  // calcul
  double inner_product(set_input*) const;
  double norm() const;

  // recorregut de features
  typedef  set<iFeature>::const_iterator const_iterator;

  const_iterator begin() const {  return sf.begin(); }
  const_iterator end() const { return sf.end(); }
  
  void print(ostream& o) const;
  void write(ostream& o) const;
};




class mlOutput {
 private:

  struct label {
    bool b;
    double weight;
    double prediction;

    label():
      b(false), weight(0.0), prediction(0.0)
    { }
  };
  
  // array of labels
  label* ml;
  

  static char _separator;
  void parse_string(string);

 protected: 
  static int _nlabels;

 public:
  static void set_separator(char);
  static void set_nlabels(int);
  static int nlabels();

  mlOutput();
  // labels joined by separator
  mlOutput(string);
  ~mlOutput();
  
  void set_label(int l, bool b) { ml[l].b = b; }
  bool belongs_to(int l) const { return ml[l].b; }
  int sign(int l) const { return ml[l].b ? +1 : -1; }

  void set_weight(int l, double w) { ml[l].weight = w; }
  double weight(int l) const { return ml[l].weight; }
  
  void set_prediction(int l, double pr) { ml[l].prediction = pr; }
  double prediction(int l) { return ml[l].prediction; }
  

};



/********************************************************************************/
/*                                                                              */
/*  example:  multilabelled example                                           */
/*                                                                              */
/********************************************************************************/

class example: public mlOutput, public input {
   
public: 
  example(string ml, string f) : mlOutput(ml), input(f) {}
  example() : mlOutput(), input() {}

  void print(ostream& o) const;

  typedef input::const_iterator feature_iterator;
};



#endif
