//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

#ifndef _LANGUAGE
#define _LANGUAGE

#include <string>
#include <list>
#include <vector>

#include "tree.h"

using namespace std;

class word; // predeclaration

////////////////////////////////////////////////////////////////
///   Class analysis stores a possible reading (lemma, PoS, probability)
///   for a word.
////////////////////////////////////////////////////////////////

class analysis {

   private:
      /// lemma
      string lemma;
      /// PoS tag
      string parole;
      /// probability of that lemma-tag given the word
      double prob;
      /// list of possible senses for that analysis
      list<string> senses;
      /// information to retokenize the word after tagging if this analysis is selected
      list<word> retok;

   public:
      /// constructor
      analysis();
      /// constructor
      analysis(const string &, const string &);
       
      void set_lemma(const string &);
      void set_parole(const string &);
      void set_prob(double);
      void set_retokenizable(const list<word> &);

      bool has_prob(void) const;  
      string get_lemma(void) const;
      string get_parole(void) const;
      string get_short_parole(const string &) const;
      double get_prob(void) const;
      bool is_retokenizable(void) const;
      list<word> get_retokenizable(void) const;

      list<string> get_senses(void) const;
      void set_senses(const list<string> &);
};


////////////////////////////////////////////////////////////////
///   Class word stores all info related to a word: 
///  form, list of analysis, list of tokens (if multiword).
////////////////////////////////////////////////////////////////

class word : public list<analysis> {
   private:
     /// lexical form
     string form;                 
     /// selected analysis (if any)
     word::iterator selected;
     /// empty list if not a multiword
     list<word> multiword;
     /// token span
     unsigned int start, finish;
     /// word form found in dictionary
     bool in_dict;
     /// Private data
     void *user;
  
   public:
      /// constructor
      word();
      /// constructor
      word(const string &);
      /// constructor
      word(const string &, const list<word> &);
      /// constructor
      word(const string &, const list<analysis> &, const list<word> &);
      /// Copy constructor
      word(const word &);
      /// assignment
      word& operator=(const word&);

      /// true iff the word has more than one analysis
      bool is_ambiguous(void) const;
      /// true iff the word is a multiword compound
      bool is_multiword(void) const;
      /// get number of words in compound
      int get_n_words_mw(void) const;
      /// get word objects that compound the multiword
      list<word> get_words_mw(void) const;
      /// get word form
      string get_form(void) const;
      /// Get the selected analysis
      analysis get_selected_analysis(void) const;
      /// Get an iterator to the selected analysis
      word::iterator selected_analysis(void) const;
      /// get lemma for the selected analysis in list
      string get_lemma(void) const;
      /// get parole for the selected analysis  
      string get_parole(void) const;
      /// get parole (short version) for the selected analysis  
      string get_short_parole(const string &) const;

      /// get sense list for the selected analysis  
      list<string> get_senses(void) const;
      /// set sense list for the selected analysis  
      void set_senses(const list<string> &);
   
      /// get token span.
      unsigned int get_span_start(void) const;
      unsigned int get_span_finish(void) const;

      /// get in_dict
      bool found_in_dict(void) const;
      /// set in_dict
      void set_found_in_dict(bool);

      /// add one analysis to current analysis list  (no duplicate check!)
      void add_analysis(const analysis &);
      /// set analysis list to one single analysis, overwriting current values
      void set_analysis(const analysis &);
      /// set analysis list, overwriting current values
      void set_analysis(const list<analysis> &);
      /// set word form
      void set_form(const string &);
      /// set token span
      void set_span(unsigned int, unsigned int);
      /// set user data
      void set_user_data(void *);
      
      /// get number of analysis in current list
      int get_n_analysis(void) const; 
      /// copy analysis list
      void copy_analysis(const word &);
      /// mark the given analysis as the selected one.
      void select_analysis(word::iterator);
      /// get list of analysis (useful for perl API)
      list<analysis> get_analysis(void) const; 
      /// get begin iterator to analysis list (useful for perl/java API)
      word::iterator analysis_begin(void);
      word::const_iterator analysis_begin(void) const;
      /// get end iterator to analysis list (useful for perl/java API)
      word::iterator analysis_end(void);
      word::const_iterator analysis_end(void) const;
};



////////////////////////////////////////////////////////////////
///   Class node stores nodes of a parse_tree
///  Each node in the tree is either a label (intermediate node)
///  or a word (leaf node)
////////////////////////////////////////////////////////////////

class node {
  protected:
    /// is the node the head of the rule?
    bool head;
    /// is the node the root of a chunk? which?
    int chunk;
    /// node label
    string label;
    /// sentence word related to the node (if leaf)
    word * w;


  public:
    /// constructors
    node();
    node(const string &);
    /// get node label
    string get_label(void) const;
    /// get node word
    word get_word(void) const;
    /// set node label
    void set_label(const string &);
    /// set node word
    void set_word(word &);
    /// find out whether node is a head
    bool is_head(void) const;
    /// set whether node is a head
    void set_head(const bool);
    /// find out whether node is a chunk
    bool is_chunk(void) const;
    /// set position of the chunk in the sentence
    void set_chunk(const int);
    /// get position of the chunk in the sentence
    int  get_chunk_ord(void) const;

  
};

////////////////////////////////////////////////////////////////
///   Class parse tree is used to store the results of parsing
////////////////////////////////////////////////////////////////

class parse_tree : public tree<node> {
    public:
      parse_tree();
      parse_tree(parse_tree::iterator p);
      parse_tree(const node &);
};


////////////////////////////////////////////////////////////////
/// class denode stores nodes of a dependency tree and
///  parse tree <-> deptree relations
////////////////////////////////////////////////////////////////

class depnode : public node {

  private:
    /// correspondinf node of the parse tree in the same sentence.
    parse_tree::iterator itree;
    /// information about categories which support the building of dependencies
    string dep_source;
    string dep_target;
    string dep_result;

  public:
    depnode();
    depnode(const string &);
    depnode(const node &);
    void set_link(const parse_tree::iterator);
    parse_tree::iterator get_link(void);
    
    void set_dep_source(const string &);
    void set_dep_target(const string &); 
    void set_dep_result(const string &); 
    string get_dep_source(void) const; 
    string get_dep_target(void) const;
    string get_dep_result(void) const;
    
};



////////////////////////////////////////////////////////////////
/// class dep_tree stores a dependency tree
////////////////////////////////////////////////////////////////

class dep_tree :  public tree<depnode> {
  public:
    dep_tree();
    dep_tree(const depnode &);
};


////////////////////////////////////////////////////////////////
///   Class sentence is just a list of words that someone
/// (the splitter) has validated it as a complete sentence.
/// It may include a parse tree.
////////////////////////////////////////////////////////////////

class sentence : public vector<word> {
 private:
  parse_tree pt;
  dep_tree dt;

 public:
  sentence();
  
  void set_parse_tree(const parse_tree &);
  parse_tree & get_parse_tree(void);
  bool is_parsed() const;
  
  dep_tree & get_dep_tree();
  void set_dep_tree(const dep_tree &);
  bool is_dep_parsed() const;

  /// get word list (useful for perl API)
  vector<word> get_words() const;
  /// get iterators to word list (useful for perl/java API)
  sentence::iterator words_begin(void);
  sentence::const_iterator words_begin(void) const;
  sentence::iterator words_end(void);
  sentence::const_iterator words_end(void) const;
};

////////////////////////////////////////////////////////////////
///   Class paragraph is just a list of sentences that someone
///  has validated it as a paragraph.
////////////////////////////////////////////////////////////////

class paragraph : public list<sentence> {};

////////////////////////////////////////////////////////////////
///   Class document is a list of paragraphs. It may have additional 
///  information (such as title)
////////////////////////////////////////////////////////////////

class document : public list<paragraph> {
  paragraph title;
};


#endif

